#!/bin/bash
MUT_FILE_PATH="/tmp/.dcismmutlogger"
ISM_MUT_LOGGER="/opt/dell/srvadmin/iSM/lib64/dcism/ismmutlogger"
DEFAULT_RPM_PARAM="install:watchdog,lclog,osinfo:remove:null,installmethod=rpm"
DEFAULT_REMOVE_RPM_PARAM="install:null:remove:watchdog,lclog,osinfo,installmethod=rpm"
DEFAULT_DEB_PARAM="install:watchdog,lclog,osinfo:remove:null,installmethod=debian"
DEFAULT_REMOVE_DEB_PARAM="install:null:remove:watchdog,lclog,osinfo,installmethod=debian"
OS_TYPE="UNKNOWN"

getOS()
{
    if [ -f /etc/redhat-release ] && [ `grep -Eci "Santiago|CentOs.* 6\..*" /etc/redhat-release` -gt 0 ]; then
        OS_TYPE="RHEL6"
    elif [ -f /etc/redhat-release ] && [ `grep -Eci "Maipo|CentOs.* 7\..*" /etc/redhat-release` -gt 0 ]; then
        OS_TYPE="RHEL7"
    # Else check if operating system is SLES.
    elif [ -f /etc/SuSE-release ]; then
        LOC_VERSION=`cat /etc/SuSE-release | grep "VERSION" | sed  -e 's#[^0-9]##g'`
        # Check if operating system is SLES11.
        if [ "${LOC_VERSION}" = "11" ]; then
                OS_TYPE="SLES11"
        elif [ "${LOC_VERSION}" = "12" ]; then
		# operating system is SLES12.
                OS_TYPE="SLES12"
        fi
    elif [ -f /etc/os-release ]; then 
		# check if operating system is UBUNTU
        . /etc/os-release
		OS=$NAME
		VER=`echo $VERSION_ID | cut -d"." -f1`
		if [ "$OS" = "Ubuntu" ] && [ "$VER" = "16" ]; then
			OS_TYPE="UBUNTU16"
		fi
		if [ "$OS" = "Ubuntu" ] && [ "$VER" = "18" ]; then
			OS_TYPE="UBUNTU18"
		fi
		#check for SLES15
		if [ "$OS" = "SLES" ] && [ "$VER" = "15" ]; then
			OS_TYPE="SLES15"
		fi
		
		# check if operating system is CentOS
		cat /etc/os-release | grep NAME | grep -i CentOS >/dev/null 2>&1
        if [ $? -eq 0 ]; then
			OS_TYPE="CENTOS"
        fi
		# check if operating system is Xen
		cat /etc/os-release | grep "XenServer"  > /dev/null
		if [ $? -eq 0 ]; then
			OS_TYPE="XEN"
		fi
		# check if operating system is Debian
		cat /etc/os-release | grep NAME | grep -i Debian >/dev/null 2>&1
        if [ $? -eq 0 ]; then
			OS_TYPE="DEBIAN"
        fi
    fi
}


if [ -f $MUT_FILE_PATH ]; then
	mut_params=`cat $MUT_FILE_PATH`
	$ISM_MUT_LOGGER $mut_params
	rm -f $MUT_FILE_PATH
else
	getOS
	if [ "$OS_TYPE" = "UBUNTU16" ] || [ "$OS_TYPE" = "UBUNTU18" ] || [ "$OS_TYPE" = "DEBIAN" ]; then
		if [ $2 -ne 0 ]; then
			$ISM_MUT_LOGGER $1 os=$OS_TYPE"."$DEFAULT_DEB_PARAM
		else
			$ISM_MUT_LOGGER $1 os=$OS_TYPE"."$DEFAULT_REMOVE_DEB_PARAM
		fi
	else
		if [ $2 -ne 0 ]; then
			$ISM_MUT_LOGGER $1 os=$OS_TYPE"."$DEFAULT_RPM_PARAM
		else
			$ISM_MUT_LOGGER $1 os=$OS_TYPE"."$DEFAULT_REMOVE_RPM_PARAM
		fi
	fi
fi
